package fer.hr.help;

import fer.hr.preprocessing.binarisation.IBinarisation;
import fer.hr.preprocessing.binarisation.OtsuBinarisation;
import fer.hr.preprocessing.greyscale.Greyscale;
import fer.hr.preprocessing.histogram.*;
import fer.hr.preprocessing.greyscale.IGreyscaleConvert;
import fer.hr.recognition.IRecognition;
import fer.hr.recognition.SimpleRecognition;
import fer.hr.segmentation.ISegmentation;
import fer.hr.segmentation.SegmentationWithBreakPoints;
import fer.hr.segmentation.SegmentationWithPresetThreshold;
import fer.hr.segmentation.SegmentationWithScaledPresetThreshold;
import javafx.application.Application;
import javafx.embed.swing.SwingFXUtils;
import javafx.geometry.Pos;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.effect.ColorInput;
import javafx.scene.effect.DropShadow;
import javafx.scene.image.*;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.FlowPane;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Text;
import javafx.stage.Stage;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.ArrayList;
import java.util.List;

import static javafx.scene.paint.Color.BLACK;
import static javafx.scene.paint.Color.RED;

/**
 * Created by jcular on 3/9/16.
 */
public class GraphicalHistogram extends Application{
    private String pathToPictures = "../test";
    private IGreyscaleConvert greyscale = new Greyscale();
    private IBinarisation binarisation = new OtsuBinarisation();
    private IHistogram pixelHistogram = new PixelHistogram();
    private IRecognition recognition = new SimpleRecognition();
    private int i = 0;
    @Override
    public void start(Stage primaryStage) throws Exception {
        File folder = new File(pathToPictures);
        File[] files = folder.listFiles();
        FlowPane flowPane = new FlowPane();
        flowPane.setVgap(10);
        flowPane.setHgap(25);
        flowPane.setAlignment(Pos.CENTER);
        flowPane.setStyle("-fx-background-color: lightgray;");

        flowPane.getChildren().clear();
        Text text1 = new Text(String.valueOf(files[0]));
        flowPane.getChildren().add(text1);
        flowPane.getChildren().addAll(drawPicture(primaryStage, files[0]));

        Scene scene = new Scene(flowPane, 1100 , 768);
        scene.setOnKeyPressed(event -> {
            if (event.getCode() == KeyCode.RIGHT) {
                i = i < files.length?i+1:i;
                flowPane.getChildren().clear();
                Text text = new Text(String.valueOf(files[i]));
                flowPane.getChildren().add(text);
                flowPane.getChildren().addAll(drawPicture(primaryStage, files[i]));
            } else if (event.getCode() == KeyCode.LEFT){
                i = i == 0?0:i-1;
                flowPane.getChildren().clear();
                Text text = new Text(String.valueOf(files[i]));
                flowPane.getChildren().add(text);
                flowPane.getChildren().addAll(drawPicture(primaryStage, files[i]));
            }
        });

        primaryStage.setScene(scene);
        primaryStage.show();


    }

    private List<Group> drawPicture(Stage primaryStage, File file) {
        BufferedImage bf = null;
        try {
            bf = ImageIO.read(file);
        } catch (IOException e) {
            e.printStackTrace();
        }
        List<Group> groups = new ArrayList<>();

        ImageView iv1 = new ImageView();
        WritableImage oldimg = new WritableImage(bf.getWidth(), bf.getHeight());
        SwingFXUtils.toFXImage(bf,oldimg);
        iv1.setImage(oldimg);

        groups.add(new Group(iv1));
        IHistogram columnSum = new ColumnSumHistogram();
        IHistogram rowSum = new RowSumHistogram();
//        ISegmentation segmentation = new SegmentationWithBreakPoints(new TopBreakPointHistogram(), new BotBreakPointHistogram(), new RightBreakPointHistogram(), new LeftBreakPointHistogram(), 150, 160, 63, 63);
        ISegmentation segmentation = new SegmentationWithScaledPresetThreshold(new ScaledHistogramDecorator(rowSum, AbstractHistogramDecorator.TYPE.WIDTH), new ScaledHistogramDecorator(columnSum, AbstractHistogramDecorator.TYPE.HEIGHT));
//        ISegmentation segmentation = new SegmentationWithPresetThreshold(rowSum, columnSum);

        greyscale.convertToGrey(bf);
        double[] hist = pixelHistogram.calculateHistogram(bf);
        double[] scaledHist = scaleHist(hist);
        bf = binarisation.convertImage(bf, hist);
        List<BufferedImage> images = segmentation.extractCharacters(bf);
//        images.add(bf);
        double[] pixelColumnSum = columnSum.calculateHistogram(bf);
        double[] pixelRowSum = rowSum.calculateHistogram(bf);
        groups.add(makeChart(pixelColumnSum, bf.getWidth()));

        StringBuilder sb = new StringBuilder();
        for (BufferedImage image : images) {
            BufferedImage image1 = image;//binarisation.convertImage(image, hist);
            ImageView iv = new ImageView();
            sb.append(recognition.recognize(image));
            WritableImage wimg = new WritableImage(image1.getWidth(), image1.getHeight());
            SwingFXUtils.toFXImage(image1,wimg);
            iv.setImage(wimg);
            groups.add(new Group(iv));
        }
//
//        if (images.size() != 10) {
//            images = new SegmentationWithBreakPoints(new TopBreakPointHistogram(), new BotBreakPointHistogram(), new RightBreakPointHistogram(), new LeftBreakPointHistogram()).extractCharacters(bf);
//            for (BufferedImage image : images) {
//                BufferedImage image1 = image;//binarisation.convertImage(image, hist);
//                ImageView iv = new ImageView();
//                sb.append(recognition.recognize(image));
//                WritableImage wimg = new WritableImage(image1.getWidth(), image1.getHeight());
//                SwingFXUtils.toFXImage(image1,wimg);
//                iv.setImage(wimg);
//                groups.add(new Group(iv));
//            }
//        }

        Text txt = new Text();
        txt.setText(sb.toString());
        groups.add(new Group(txt));

        return groups;
    }

    private double[] scaleHist(double[] hist) {
        double sum = 0;
        for (int j = 0; j < hist.length; j++) {
            sum += hist[j];
        }
        double[] newHist = new double[hist.length];
        for (int j = 0; j < hist.length; j++) {
            newHist[j] = hist[j] / sum;
        }
        return newHist;
    }

    private Group makeChart(double[] values, double height) {
        Group Chart = new Group();

        for(int i = 0; i < values.length; i++) {
            Chart.getChildren().add(drawBar(i,values[i], height));
        }

        return Chart;
    }

    private Rectangle drawBar(double xValue, double yValue, double height) {
        Rectangle rect = new Rectangle(0,yValue);
        rect.setX(xValue);
        rect.setY((height-yValue));
        rect.setStroke(BLACK);
        return rect;
    }
}
