package fer.hr.pipeline;

import fer.hr.preprocessing.binarisation.IBinarisation;
import fer.hr.preprocessing.binarisation.OtsuBinarisation;
import fer.hr.preprocessing.greyscale.Greyscale;
import fer.hr.preprocessing.greyscale.IGreyscaleConvert;
import fer.hr.preprocessing.histogram.*;
import fer.hr.recognition.IRecognition;
import fer.hr.recognition.SimpleRecognition;
import fer.hr.segmentation.ISegmentation;
import fer.hr.segmentation.SegmentationWithBreakPoints;
import fer.hr.segmentation.SegmentationWithPresetThreshold;
import fer.hr.segmentation.SegmentationWithScaledPresetThreshold;

import java.io.*;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Created by jcular on 5/8/16.
 */
public class Pipeline {
    private static final String picturesPath = "../seven-segment";

    public static void main(String[] args) {

        Map<String, String> mapOfResults = readIndexFile();
        IHistogram rowSum = new RowSumHistogram();
        IHistogram columnSum = new ColumnSumHistogram();
        ISegmentation segmentation = new SegmentationWithScaledPresetThreshold(new ScaledHistogramDecorator(rowSum, AbstractHistogramDecorator.TYPE.WIDTH), new ScaledHistogramDecorator(columnSum, AbstractHistogramDecorator.TYPE.HEIGHT));
//        ISegmentation segmentation = new SegmentationWithPresetThreshold(rowSum, columnSum);
//        ISegmentation segmentation = new SegmentationWithBreakPoints(new TopBreakPointHistogram(), new BotBreakPointHistogram(), new RightBreakPointHistogram(), new LeftBreakPointHistogram(), 150, 160, 60, 60);
        IHistogram pixelHistogram = new PixelHistogram();
        IBinarisation otsuBinarisation = new OtsuBinarisation();
        IRecognition recognition = new SimpleRecognition();
        IGreyscaleConvert greyscaleConvert = new Greyscale();
        startRecognition(segmentation, mapOfResults, pixelHistogram, otsuBinarisation, recognition, greyscaleConvert);
    }

    private static int startRecognition(ISegmentation segmentation, Map<String, String> mapOfResults, IHistogram pixelHistogram, IBinarisation otsuBinarisation, IRecognition recognition, IGreyscaleConvert greyscaleConvert) {
        PipelineContext strategy = new PipelineContext(segmentation, otsuBinarisation, recognition, pixelHistogram, greyscaleConvert);

        int badPictureRead = 0;
        int badCharRead = 0;
        int charactersRead = 0;
        int badSegmentation = 0;
        Integer[][] sTk = new Integer[11][11];
        for (int i = 0; i < sTk.length; i++) {
            for (int j = 0; j < sTk.length; j++) {
                sTk[i][j] = 0;
            }
        }
        File dir = new File(picturesPath);
        File[] directoryListing = dir.listFiles();
        int chracters = 0;
        try {
            if (directoryListing != null) {
                for (File child : directoryListing) {
                    String pictureName = String.valueOf(child).replace(picturesPath+"/", "");
                    String value = mapOfResults.get(pictureName);
                    String recognizedSequence = strategy.recognizeNumbers(String.valueOf(child.toPath()));
                    if (recognizedSequence.length() == 10) {
                        for (int i = 0; i < value.length() && i < recognizedSequence.length(); i++) {
                            chracters++;
                            sTk[getValue(value.charAt(i))][getValue(recognizedSequence.charAt(i))]++;
                            if (value.charAt(i) != recognizedSequence.charAt(i)) {
                                badCharRead++;
                            }
                            charactersRead++;
                        }
                    } else {
                        System.out.println(child.toString());
                        badSegmentation++;
                    }

                    if (!recognizedSequence.equals(value)) {
                        badPictureRead++;
                        System.out.println("Recognized " + recognizedSequence + " Expected " + value + " File:" + child.toString());
                    }
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }

//        System.out.println(chracters);
//        printConfusion(sTk);

        DecimalFormat fmt = new DecimalFormat("###.##");
        StringBuilder sb = new StringBuilder();
        sb.append("Bad pictures read = ").append(badPictureRead);
        sb.append(" Good pictures read = ").append(directoryListing.length - badPictureRead);
        sb.append(" Segmentation fault = ").append(badSegmentation);
        sb.append("\nPicture error = ").append(fmt.format(badPictureRead * 1.0/directoryListing.length*100));
        sb.append(" Character error = ").append(fmt.format(badCharRead * 1.0/charactersRead*100));
        sb.append(" Picture good = ").append(fmt.format(100 - badPictureRead * 1.0/directoryListing.length*100));
        sb.append(" Character good = ").append(fmt.format(100 - badCharRead * 1.0/charactersRead*100));
        sb.append("\nBad characters = ").append(badCharRead);
        sb.append(" Good characters = ").append(charactersRead - badCharRead);

        System.out.println(sb);
        return badSegmentation+badPictureRead;

    }

    private static void printConfusion(Integer[][] sTk) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < sTk.length; i++) {
            sb.append(i).append("&");
            sb.append(String.join("&", Arrays.asList(sTk[i]).stream().map(c -> Integer.toString(c)).collect(Collectors.toList())));

            sb.append("\\\\");
            sb.append("\n");
        }

        System.out.println(sb.toString());
    }

    private static int getValue(char c) {
        if (c == '?') {
            return 10;
        } else {
            return c - '0';
        }
    }

    private static void findBestThreshold() {
        Map<String, String> mapOfResults = readIndexFile();
        IHistogram rowSum = new RowSumHistogram();
//        rowSum = new ScaledHistogramDecorator(rowSum);
        IHistogram columnSum = new ColumnSumHistogram();
//        columnSum = new ScaledHistogramDecorator(columnSum);
//        ISegmentation segmentation = new SegmentationWithScaledPresetThreshold(rowSum, columnSum);
        IHistogram pixelHistogram = new PixelHistogram();
        IBinarisation otsuBinarisation = new OtsuBinarisation();
        IRecognition recognition = new SimpleRecognition();
        IGreyscaleConvert greyscaleConvert = new Greyscale();

        int bestLeft= 0;
        int bestRight = 0;
        int bestTop = 0;
        int bestBot = 0;

        int max = 522;
        int cntr = 0;
        for (int i = 150; i < 200; i++) {
//            for (int j = 120; j < 200; j++) {
//                for (int k = 20; k < 40; k++) {
                    for (int l = 25; l < 40; l++) {
                        System.out.println(cntr++);
                        ISegmentation segmentation = new SegmentationWithBreakPoints(new TopBreakPointHistogram(), new BotBreakPointHistogram(), new RightBreakPointHistogram(), new LeftBreakPointHistogram(), i, i, l, l);
                        int current = startRecognition(segmentation, mapOfResults, pixelHistogram, otsuBinarisation, recognition, greyscaleConvert);
                        System.out.println("current:" + current + " i j k l " + i + " " + i + " " + l + " " + l);
                        if (current < max) {
                            max = current;
                            bestLeft = i;
//                            bestRight = j;
//                            bestTop = k;
                            bestBot = l;
                        }
//                    }
//                }
            }
        }

        System.out.println(bestLeft + " " + bestRight + " " + bestTop +" " + bestBot);
    }

    private static Map<String, String> readIndexFile() {
        Map<String, String> map = new HashMap<>();
        try {
            BufferedReader br = new BufferedReader(new FileReader("index.txt"));
            while (br.ready()) {
                String line = br.readLine();
                String name = line.split("\t")[0].trim();
                String value = line.split("\t")[1].trim();
                map.put(name, value);
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return map;
    }
}
