package fer.hr.preprocessing.binarisation;

import java.awt.image.BufferedImage;

/**
 * Created by jcular on 3/9/16.
 */
public class OtsuBinarisation implements IBinarisation {

    @Override
    public BufferedImage convertImage(BufferedImage oldImage, double[] histogram) {
        BufferedImage image = new BufferedImage(oldImage.getWidth(), oldImage.getHeight(), oldImage.getType());
        int threshold = findThreshold(histogram, oldImage.getHeight() * oldImage.getWidth());
//        System.out.println(threshold);
        binarizeImage(oldImage, image, threshold);
        return image;
    }

    private int findThreshold(double[] histogram, int numOfPixels) {
        double n = 0;

        int threshold = 0;

        for (int t = 0; t < histogram.length; t++) {
            double classProbability0 = classProbability(histogram,t, numOfPixels);
            double classProbability1 = 1 - classProbability0;
            double classMean0 = classMean(histogram, t + 1, 0);
            double classMean1 = classMean(histogram, histogram.length, t+1);

            double sigmaB = calculateSigmaB(classProbability0,classProbability1,classMean0,classMean1);

            if(sigmaB > n) {
                n = sigmaB;
                threshold = t;
            }
        }

        return threshold;
    }

    private void binarizeImage(BufferedImage oldImage, BufferedImage newImage, int threshold) {
        int width = newImage.getWidth();
        int height = newImage.getHeight();

        for (int i = 0; i < height; i++) {
            for (int j = 0; j < width; j++) {
                int pixelIntensity = oldImage.getRGB(j, i) & 0xFF;
                if (pixelIntensity > threshold) {
                    newImage.setRGB(j, i, 0xFFFFFF);
                } else {
                    newImage.setRGB(j, i, 0x000000);
                }
            }
        }
    }

    private double classMean(double[] histogram,int endt, int startt) {
        double sum = 0;
        double sumB = 0;
        for(int i=startt; i < endt; i++){
            sum += i*histogram[i];
            sumB += histogram[i];
        }
        return sum/sumB;
    }

    private double calculateSigmaB(double classProbability0, double classProbability1, double classMean0, double classMean1) {
        return classProbability0 * classProbability1 * (classMean0-classMean1) * (classMean0-classMean1);
    }

    private double classProbability(double[] histogram, int t, int numOfPixels) {
        double sum = 0;
        for (int i = 0; i <= t; i++) {
            sum += histogram[i];
        }
        return sum/(double) numOfPixels;
    }
}
