package fer.hr.recognition;

import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by jcular on 5/10/16.
 */
public class SimpleRecognition implements IRecognition {
    protected Map<String, Integer> encoding;
    private final double FILLTHRESHOLD = 0.285;

    public SimpleRecognition() {
        loadMap();
    }

    private void loadMap() {
        encoding = new HashMap<>();
        try {
            BufferedReader bufferedReader = new BufferedReader(new FileReader("/home/jcular/git/zavrsni/src/fer/hr/recognition/code"));

            while (bufferedReader.ready()) {
                String line = bufferedReader.readLine();
                String value = line.split(" ")[0].trim();
                int number = Integer.parseInt(line.split(" ")[1]);
                encoding.put(value, number);
            }

            bufferedReader.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public int recognize(BufferedImage image) {
        int[] positionsX = findPositionsX(image);
        int[] positionsY = findPositionsY(image);
        int[] heights = findHeights(positionsY, image);
        int[] widths = findWidths(positionsX, image);

        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < heights.length; i++) {
            sb.append(isSegmentFilled(positionsX[i], positionsY[i], heights[i], widths[i], image));
        }
        if (!encoding.containsKey(sb.toString())) {
            return -1;
        } else {
            return encoding.get(sb.toString());
        }
    }

    private int[] findWidths(int[] positionsX, BufferedImage image) {
        int width1 = positionsX[2] - positionsX[0];
        int width2 = positionsX[7] - positionsX[1];
        int width3 = positionsX[0] - positionsX[4];
        return new int[]{width1, width2, width2, width1, width3, width3, width1};
//        int width1 = positionsX[2] - positionsX[0];
//        int width2 = image.getWidth() - positionsX[1];
//        int width3 = positionsX[0] - positionsX[4];
//        return new int[]{width1, width2, width2, width1, width3, width3, width1};
    }

    private int[] findHeights(int[] positionsY, BufferedImage image) {
        int height1 = positionsY[1] - positionsY[0];
        int height2 = positionsY[6] - positionsY[1];
        int height3 = positionsY[3] - positionsY[2];
        int height4 = positionsY[7] - positionsY[3];
        int height5 = positionsY[2] - positionsY[6];
        return new int[]{height1, height2, height3, height4, height3, height2, height5};
//        int height1 = positionsY[1];
//        int height2 = positionsY[6] - positionsY[1];
//        int height3 = positionsY[3] - positionsY[2];
//        int height4 = image.getHeight() - positionsY[3];
//        int height5 = positionsY[2] - positionsY[6];
//        return new int[]{height1, height2, height3, height4, height3, height2, height5};
    }

    private int[] findPositionsX(BufferedImage image) {
        int y = image.getHeight()*2/3;
        int x2 = image.getWidth()/2;
        int x3 = image.getWidth()/2;
        int x1 = 0;
        int x4 = image.getWidth()-1;

        while(x2 > 0 && (image.getRGB(x2,y) & 0xFF) != 0){
            x2--;
        }

        while(x3 < image.getWidth() && (image.getRGB(x3,y) & 0xFF) != 0){
            x3++;
        }


        while(x1 < image.getWidth() && (image.getRGB(x1,y) & 0xFF) != 0){
            x1++;
        }


        while(x4 > 0 && (image.getRGB(x4,y) & 0xFF) != 0){
            x4--;
        }

        return new int[]{x2, x3, x3, x2, x1, x1, x2, x4};
    }

    private int[] findPositionsY(BufferedImage image) {
        int x = image.getWidth()/2;
        int y1 = 0;
        int y2 = image.getHeight()*1/3;
        int y3 = image.getHeight()*1/3;
        int y4 = image.getHeight()*2/3;
        int y5 = image.getHeight()*2/3;
        int y6 = image.getHeight()-1;

        while(y1 < image.getHeight() && (image.getRGB(x,y1) & 0xFF) != 0){
            y1++;
        }

        while(y2 > 0 && (image.getRGB(x,y2) & 0xFF) != 0){
            y2--;
        }

        while(y3 < image.getHeight()/2 && (image.getRGB(x,y3) & 0xFF) != 0){
            y3++;
        }

        while(y4 > image.getHeight()/2 && (image.getRGB(x,y4) & 0xFF) != 0){
            y4--;
        }

        while(y5 < image.getHeight() && (image.getRGB(x,y5) & 0xFF) != 0){
            y5++;
        }

        while(y6 > 0 && (image.getRGB(x,y6) & 0xFF) != 0){
            y6--;
        }

        return new int[]{y1, y2, y4, y5, y4, y2, y3, y6};
    }

    public int isSegmentFilled(int x, int y,int width, int height, BufferedImage image) {
        int count = 0;
        for (int i = x; i < x + width && i < image.getWidth(); i++) {
            for (int j = y; j < y + height && j < image.getHeight(); j++) {
                if ((image.getRGB(i,j) & 0xFF) == 0) {
                    count++;
                }
            }
        }
        return (count*1.0/(width*height)) > FILLTHRESHOLD ? 1 : 0;
    }
}
