package fer.hr.segmentation;

import fer.hr.preprocessing.histogram.IHistogram;

import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by jcular on 4/3/16.
 */
public class SegmentationWithBreakPoints implements ISegmentation {
    private double leftMin;
    private double rightMin;
    private double topMin;
    private double botMin;

    private IHistogram topHistogram;
    private IHistogram botHistogram;
    private IHistogram rightHistogram;
    private IHistogram leftHistogram;

    public SegmentationWithBreakPoints(IHistogram topHistogram, IHistogram botHistogram,
                                       IHistogram rightHistogram, IHistogram leftHistogram,
                                       double leftMin, double rightMin, double topMin, double botMin) {
        this.topHistogram = topHistogram;
        this.botHistogram = botHistogram;
        this.rightHistogram = rightHistogram;
        this.leftHistogram = leftHistogram;
        this.leftMin = leftMin;
        this.rightMin = rightMin;
        this.topMin = topMin;
        this.botMin = botMin;
    }

    public SegmentationWithBreakPoints(IHistogram topHistogram, IHistogram botHistogram,
                                       IHistogram rightHistogram, IHistogram leftHistogram) {
        this(topHistogram, botHistogram, rightHistogram, leftHistogram, 150, 150, 25, 25);
    }

    @Override
    public List<BufferedImage> extractCharacters(BufferedImage image) {

        double[] leftH = leftHistogram.calculateHistogram(image);
        double[] rightH = rightHistogram.calculateHistogram(image);
        int y1 = 0, y2 = 0;

        while (y1 < image.getHeight() && y2 < image.getHeight()) {
            if (leftH[y1] < leftMin && rightH[y1] < rightMin && y1 >= y2) {
                y2 = y1 + 1;
            } else if (y2 > y1 && leftH[y2] < leftMin && rightH[y2] < rightMin) {
                y2++;
            } else if (y2 > y1 && !(leftH[y2] < leftMin && rightH[y2] < rightMin)) {
                break;
            } else {
                y1++;
            }
        }

        if (y2 < y1 ) {
            y1 = 0;
            y2 = image.getHeight();
        }

        BufferedImage horizontalCut = image.getSubimage(0, y1, image.getWidth(), Math.abs(y2 - y1));

        double[] topH = topHistogram.calculateHistogram(horizontalCut);
        double[] botH = botHistogram.calculateHistogram(horizontalCut);

        List<BufferedImage> characterImages = new ArrayList<>();
        int x1 = 0, x2 = 0;

        while (x1 < horizontalCut.getWidth() && x2 < horizontalCut.getWidth()) {
            if (topH[x1] < topMin && botH[x1] < botMin && x1 >= x2) {
                x2 = x1 + 1;
            } else if (x2 > x1 && topH[x2] < topMin && botH[x2] < botMin) {
                x2++;
            } else if (x2 > x1 && !(topH[x2] < topMin && botH[x2] < botMin)) {
                characterImages.add(horizontalCut.getSubimage(x1, 0, x2 - x1, horizontalCut.getHeight()));
                x1 = x2 + 1;
            } else {
                x1++;
            }
        }
        return characterImages;
    }
}
